# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._test_helpers import APIResourceTestHelpers
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, Union, cast, overload
from typing_extensions import Literal, Type, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.issuing._cardholder import Cardholder
    from stripe.issuing._personalization_design import PersonalizationDesign
    from stripe.params.issuing._card_create_params import CardCreateParams
    from stripe.params.issuing._card_deliver_card_params import (
        CardDeliverCardParams,
    )
    from stripe.params.issuing._card_fail_card_params import CardFailCardParams
    from stripe.params.issuing._card_list_params import CardListParams
    from stripe.params.issuing._card_modify_params import CardModifyParams
    from stripe.params.issuing._card_retrieve_params import CardRetrieveParams
    from stripe.params.issuing._card_return_card_params import (
        CardReturnCardParams,
    )
    from stripe.params.issuing._card_ship_card_params import CardShipCardParams
    from stripe.params.issuing._card_submit_card_params import (
        CardSubmitCardParams,
    )


class Card(
    CreateableAPIResource["Card"],
    ListableAPIResource["Card"],
    UpdateableAPIResource["Card"],
):
    """
    You can [create physical or virtual cards](https://docs.stripe.com/issuing) that are issued to cardholders.
    """

    OBJECT_NAME: ClassVar[Literal["issuing.card"]] = "issuing.card"

    class LatestFraudWarning(StripeObject):
        started_at: Optional[int]
        """
        Timestamp of the most recent fraud warning.
        """
        type: Optional[
            Union[
                Literal[
                    "card_testing_exposure",
                    "fraud_dispute_filed",
                    "third_party_reported",
                    "user_indicated_fraud",
                ],
                str,
            ]
        ]
        """
        The type of fraud warning that most recently took place on this card. This field updates with every new fraud warning, so the value changes over time. If populated, cancel and reissue the card.
        """

    class LifecycleControls(StripeObject):
        class CancelAfter(StripeObject):
            payment_count: int
            """
            The card is automatically cancelled when it makes this number of non-zero payment authorizations and transactions. The count includes penny authorizations, but doesn't include non-payment actions, such as authorization advice.
            """

        cancel_after: CancelAfter
        _inner_class_types = {"cancel_after": CancelAfter}

    class Shipping(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: Optional[str]
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """

        class AddressValidation(StripeObject):
            class NormalizedAddress(StripeObject):
                city: Optional[str]
                """
                City, district, suburb, town, or village.
                """
                country: Optional[str]
                """
                Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                """
                line1: Optional[str]
                """
                Address line 1, such as the street, PO Box, or company name.
                """
                line2: Optional[str]
                """
                Address line 2, such as the apartment, suite, unit, or building.
                """
                postal_code: Optional[str]
                """
                ZIP or postal code.
                """
                state: Optional[str]
                """
                State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                """

            mode: Union[
                Literal[
                    "disabled",
                    "normalization_only",
                    "validation_and_normalization",
                ],
                str,
            ]
            """
            The address validation capabilities to use.
            """
            normalized_address: Optional[NormalizedAddress]
            """
            The normalized shipping address.
            """
            result: Optional[
                Union[
                    Literal[
                        "indeterminate",
                        "likely_deliverable",
                        "likely_undeliverable",
                    ],
                    str,
                ]
            ]
            """
            The validation result for the shipping address.
            """
            _inner_class_types = {"normalized_address": NormalizedAddress}

        class Customs(StripeObject):
            eori_number: Optional[str]
            """
            A registration number used for customs in Europe. See [https://www.gov.uk/eori](https://www.gov.uk/eori) for the UK and [https://ec.europa.eu/taxation_customs/business/customs-procedures-import-and-export/customs-procedures/economic-operators-registration-and-identification-number-eori_en](https://ec.europa.eu/taxation_customs/business/customs-procedures-import-and-export/customs-procedures/economic-operators-registration-and-identification-number-eori_en) for the EU.
            """

        address: Address
        address_validation: Optional[AddressValidation]
        """
        Address validation details for the shipment.
        """
        business_name: Optional[str]
        """
        The name of the business at the shipping address, used on the shipping label to ensure delivery when the card is shipped to a cardholder's workplace.
        """
        carrier: Optional[
            Union[
                Literal["correos", "dhl", "fedex", "royal_mail", "usps"], str
            ]
        ]
        """
        The delivery company that shipped a card.
        """
        customs: Optional[Customs]
        """
        Additional information that may be required for clearing customs.
        """
        eta: Optional[int]
        """
        A unix timestamp representing a best estimate of when the card will be delivered.
        """
        name: str
        """
        Recipient name.
        """
        phone_number: Optional[str]
        """
        The phone number of the receiver of the shipment. Our courier partners will use this number to contact you in the event of card delivery issues. For individual shipments to the EU/UK, if this field is empty, we will provide them with the phone number provided when the cardholder was initially created.
        """
        require_signature: Optional[bool]
        """
        Whether a signature is required for card delivery. This feature is only supported for US users. Standard shipping service does not support signature on delivery. The default value for standard shipping service is false and for express and priority services is true.
        """
        service: Union[Literal["express", "priority", "standard"], str]
        """
        Shipment service, such as `standard` or `express`.
        """
        status: Optional[
            Union[
                Literal[
                    "canceled",
                    "delivered",
                    "failure",
                    "pending",
                    "returned",
                    "shipped",
                    "submitted",
                ],
                str,
            ]
        ]
        """
        The delivery status of the card.
        """
        tracking_number: Optional[str]
        """
        A tracking number for a card shipment.
        """
        tracking_url: Optional[str]
        """
        A link to the shipping carrier's site where you can view detailed information about a card shipment.
        """
        type: Union[Literal["bulk", "individual"], str]
        """
        Packaging options.
        """
        _inner_class_types = {
            "address": Address,
            "address_validation": AddressValidation,
            "customs": Customs,
        }

    class SpendingControls(StripeObject):
        class SpendingLimit(StripeObject):
            amount: int
            """
            Maximum amount allowed to spend per interval. This amount is in the card's currency and in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
            """
            categories: Optional[
                List[
                    Union[
                        Literal[
                            "ac_refrigeration_repair",
                            "accounting_bookkeeping_services",
                            "advertising_services",
                            "agricultural_cooperative",
                            "airlines_air_carriers",
                            "airports_flying_fields",
                            "ambulance_services",
                            "amusement_parks_carnivals",
                            "antique_reproductions",
                            "antique_shops",
                            "aquariums",
                            "architectural_surveying_services",
                            "art_dealers_and_galleries",
                            "artists_supply_and_craft_shops",
                            "auto_and_home_supply_stores",
                            "auto_body_repair_shops",
                            "auto_paint_shops",
                            "auto_service_shops",
                            "automated_cash_disburse",
                            "automated_fuel_dispensers",
                            "automobile_associations",
                            "automotive_parts_and_accessories_stores",
                            "automotive_tire_stores",
                            "bail_and_bond_payments",
                            "bakeries",
                            "bands_orchestras",
                            "barber_and_beauty_shops",
                            "betting_casino_gambling",
                            "bicycle_shops",
                            "billiard_pool_establishments",
                            "boat_dealers",
                            "boat_rentals_and_leases",
                            "book_stores",
                            "books_periodicals_and_newspapers",
                            "bowling_alleys",
                            "bus_lines",
                            "business_secretarial_schools",
                            "buying_shopping_services",
                            "cable_satellite_and_other_pay_television_and_radio",
                            "camera_and_photographic_supply_stores",
                            "candy_nut_and_confectionery_stores",
                            "car_and_truck_dealers_new_used",
                            "car_and_truck_dealers_used_only",
                            "car_rental_agencies",
                            "car_washes",
                            "carpentry_services",
                            "carpet_upholstery_cleaning",
                            "caterers",
                            "charitable_and_social_service_organizations_fundraising",
                            "chemicals_and_allied_products",
                            "child_care_services",
                            "childrens_and_infants_wear_stores",
                            "chiropodists_podiatrists",
                            "chiropractors",
                            "cigar_stores_and_stands",
                            "civic_social_fraternal_associations",
                            "cleaning_and_maintenance",
                            "clothing_rental",
                            "colleges_universities",
                            "commercial_equipment",
                            "commercial_footwear",
                            "commercial_photography_art_and_graphics",
                            "commuter_transport_and_ferries",
                            "computer_network_services",
                            "computer_programming",
                            "computer_repair",
                            "computer_software_stores",
                            "computers_peripherals_and_software",
                            "concrete_work_services",
                            "construction_materials",
                            "consulting_public_relations",
                            "correspondence_schools",
                            "cosmetic_stores",
                            "counseling_services",
                            "country_clubs",
                            "courier_services",
                            "court_costs",
                            "credit_reporting_agencies",
                            "cruise_lines",
                            "dairy_products_stores",
                            "dance_hall_studios_schools",
                            "dating_escort_services",
                            "dentists_orthodontists",
                            "department_stores",
                            "detective_agencies",
                            "digital_goods_applications",
                            "digital_goods_games",
                            "digital_goods_large_volume",
                            "digital_goods_media",
                            "direct_marketing_catalog_merchant",
                            "direct_marketing_combination_catalog_and_retail_merchant",
                            "direct_marketing_inbound_telemarketing",
                            "direct_marketing_insurance_services",
                            "direct_marketing_other",
                            "direct_marketing_outbound_telemarketing",
                            "direct_marketing_subscription",
                            "direct_marketing_travel",
                            "discount_stores",
                            "doctors",
                            "door_to_door_sales",
                            "drapery_window_covering_and_upholstery_stores",
                            "drinking_places",
                            "drug_stores_and_pharmacies",
                            "drugs_drug_proprietaries_and_druggist_sundries",
                            "dry_cleaners",
                            "durable_goods",
                            "duty_free_stores",
                            "eating_places_restaurants",
                            "educational_services",
                            "electric_razor_stores",
                            "electric_vehicle_charging",
                            "electrical_parts_and_equipment",
                            "electrical_services",
                            "electronics_repair_shops",
                            "electronics_stores",
                            "elementary_secondary_schools",
                            "emergency_services_gcas_visa_use_only",
                            "employment_temp_agencies",
                            "equipment_rental",
                            "exterminating_services",
                            "family_clothing_stores",
                            "fast_food_restaurants",
                            "financial_institutions",
                            "fines_government_administrative_entities",
                            "fireplace_fireplace_screens_and_accessories_stores",
                            "floor_covering_stores",
                            "florists",
                            "florists_supplies_nursery_stock_and_flowers",
                            "freezer_and_locker_meat_provisioners",
                            "fuel_dealers_non_automotive",
                            "funeral_services_crematories",
                            "furniture_home_furnishings_and_equipment_stores_except_appliances",
                            "furniture_repair_refinishing",
                            "furriers_and_fur_shops",
                            "general_services",
                            "gift_card_novelty_and_souvenir_shops",
                            "glass_paint_and_wallpaper_stores",
                            "glassware_crystal_stores",
                            "golf_courses_public",
                            "government_licensed_horse_dog_racing_us_region_only",
                            "government_licensed_online_casions_online_gambling_us_region_only",
                            "government_owned_lotteries_non_us_region",
                            "government_owned_lotteries_us_region_only",
                            "government_services",
                            "grocery_stores_supermarkets",
                            "hardware_equipment_and_supplies",
                            "hardware_stores",
                            "health_and_beauty_spas",
                            "hearing_aids_sales_and_supplies",
                            "heating_plumbing_a_c",
                            "hobby_toy_and_game_shops",
                            "home_supply_warehouse_stores",
                            "hospitals",
                            "hotels_motels_and_resorts",
                            "household_appliance_stores",
                            "industrial_supplies",
                            "information_retrieval_services",
                            "insurance_default",
                            "insurance_underwriting_premiums",
                            "intra_company_purchases",
                            "jewelry_stores_watches_clocks_and_silverware_stores",
                            "landscaping_services",
                            "laundries",
                            "laundry_cleaning_services",
                            "legal_services_attorneys",
                            "luggage_and_leather_goods_stores",
                            "lumber_building_materials_stores",
                            "manual_cash_disburse",
                            "marinas_service_and_supplies",
                            "marketplaces",
                            "masonry_stonework_and_plaster",
                            "massage_parlors",
                            "medical_and_dental_labs",
                            "medical_dental_ophthalmic_and_hospital_equipment_and_supplies",
                            "medical_services",
                            "membership_organizations",
                            "mens_and_boys_clothing_and_accessories_stores",
                            "mens_womens_clothing_stores",
                            "metal_service_centers",
                            "miscellaneous",
                            "miscellaneous_apparel_and_accessory_shops",
                            "miscellaneous_auto_dealers",
                            "miscellaneous_business_services",
                            "miscellaneous_food_stores",
                            "miscellaneous_general_merchandise",
                            "miscellaneous_general_services",
                            "miscellaneous_home_furnishing_specialty_stores",
                            "miscellaneous_publishing_and_printing",
                            "miscellaneous_recreation_services",
                            "miscellaneous_repair_shops",
                            "miscellaneous_specialty_retail",
                            "mobile_home_dealers",
                            "motion_picture_theaters",
                            "motor_freight_carriers_and_trucking",
                            "motor_homes_dealers",
                            "motor_vehicle_supplies_and_new_parts",
                            "motorcycle_shops_and_dealers",
                            "motorcycle_shops_dealers",
                            "music_stores_musical_instruments_pianos_and_sheet_music",
                            "news_dealers_and_newsstands",
                            "non_fi_money_orders",
                            "non_fi_stored_value_card_purchase_load",
                            "nondurable_goods",
                            "nurseries_lawn_and_garden_supply_stores",
                            "nursing_personal_care",
                            "office_and_commercial_furniture",
                            "opticians_eyeglasses",
                            "optometrists_ophthalmologist",
                            "orthopedic_goods_prosthetic_devices",
                            "osteopaths",
                            "package_stores_beer_wine_and_liquor",
                            "paints_varnishes_and_supplies",
                            "parking_lots_garages",
                            "passenger_railways",
                            "pawn_shops",
                            "pet_shops_pet_food_and_supplies",
                            "petroleum_and_petroleum_products",
                            "photo_developing",
                            "photographic_photocopy_microfilm_equipment_and_supplies",
                            "photographic_studios",
                            "picture_video_production",
                            "piece_goods_notions_and_other_dry_goods",
                            "plumbing_heating_equipment_and_supplies",
                            "political_organizations",
                            "postal_services_government_only",
                            "precious_stones_and_metals_watches_and_jewelry",
                            "professional_services",
                            "public_warehousing_and_storage",
                            "quick_copy_repro_and_blueprint",
                            "railroads",
                            "real_estate_agents_and_managers_rentals",
                            "record_stores",
                            "recreational_vehicle_rentals",
                            "religious_goods_stores",
                            "religious_organizations",
                            "roofing_siding_sheet_metal",
                            "secretarial_support_services",
                            "security_brokers_dealers",
                            "service_stations",
                            "sewing_needlework_fabric_and_piece_goods_stores",
                            "shoe_repair_hat_cleaning",
                            "shoe_stores",
                            "small_appliance_repair",
                            "snowmobile_dealers",
                            "special_trade_services",
                            "specialty_cleaning",
                            "sporting_goods_stores",
                            "sporting_recreation_camps",
                            "sports_and_riding_apparel_stores",
                            "sports_clubs_fields",
                            "stamp_and_coin_stores",
                            "stationary_office_supplies_printing_and_writing_paper",
                            "stationery_stores_office_and_school_supply_stores",
                            "swimming_pools_sales",
                            "t_ui_travel_germany",
                            "tailors_alterations",
                            "tax_payments_government_agencies",
                            "tax_preparation_services",
                            "taxicabs_limousines",
                            "telecommunication_equipment_and_telephone_sales",
                            "telecommunication_services",
                            "telegraph_services",
                            "tent_and_awning_shops",
                            "testing_laboratories",
                            "theatrical_ticket_agencies",
                            "timeshares",
                            "tire_retreading_and_repair",
                            "tolls_bridge_fees",
                            "tourist_attractions_and_exhibits",
                            "towing_services",
                            "trailer_parks_campgrounds",
                            "transportation_services",
                            "travel_agencies_tour_operators",
                            "truck_stop_iteration",
                            "truck_utility_trailer_rentals",
                            "typesetting_plate_making_and_related_services",
                            "typewriter_stores",
                            "u_s_federal_government_agencies_or_departments",
                            "uniforms_commercial_clothing",
                            "used_merchandise_and_secondhand_stores",
                            "utilities",
                            "variety_stores",
                            "veterinary_services",
                            "video_amusement_game_supplies",
                            "video_game_arcades",
                            "video_tape_rental_stores",
                            "vocational_trade_schools",
                            "watch_jewelry_repair",
                            "welding_repair",
                            "wholesale_clubs",
                            "wig_and_toupee_stores",
                            "wires_money_orders",
                            "womens_accessory_and_specialty_shops",
                            "womens_ready_to_wear_stores",
                            "wrecking_and_salvage_yards",
                        ],
                        str,
                    ]
                ]
            ]
            """
            Array of strings containing [categories](https://docs.stripe.com/api#issuing_authorization_object-merchant_data-category) this limit applies to. Omitting this field will apply the limit to all categories.
            """
            interval: Union[
                Literal[
                    "all_time",
                    "daily",
                    "monthly",
                    "per_authorization",
                    "weekly",
                    "yearly",
                ],
                str,
            ]
            """
            Interval (or event) to which the amount applies.
            """

        allowed_card_presences: Optional[
            List[Union[Literal["not_present", "present"], str]]
        ]
        """
        Array of card presence statuses from which authorizations will be allowed. Possible options are `present`, `not_present`. All other statuses will be blocked. Cannot be set with `blocked_card_presences`. Provide an empty value to unset this control.
        """
        allowed_categories: Optional[
            List[
                Union[
                    Literal[
                        "ac_refrigeration_repair",
                        "accounting_bookkeeping_services",
                        "advertising_services",
                        "agricultural_cooperative",
                        "airlines_air_carriers",
                        "airports_flying_fields",
                        "ambulance_services",
                        "amusement_parks_carnivals",
                        "antique_reproductions",
                        "antique_shops",
                        "aquariums",
                        "architectural_surveying_services",
                        "art_dealers_and_galleries",
                        "artists_supply_and_craft_shops",
                        "auto_and_home_supply_stores",
                        "auto_body_repair_shops",
                        "auto_paint_shops",
                        "auto_service_shops",
                        "automated_cash_disburse",
                        "automated_fuel_dispensers",
                        "automobile_associations",
                        "automotive_parts_and_accessories_stores",
                        "automotive_tire_stores",
                        "bail_and_bond_payments",
                        "bakeries",
                        "bands_orchestras",
                        "barber_and_beauty_shops",
                        "betting_casino_gambling",
                        "bicycle_shops",
                        "billiard_pool_establishments",
                        "boat_dealers",
                        "boat_rentals_and_leases",
                        "book_stores",
                        "books_periodicals_and_newspapers",
                        "bowling_alleys",
                        "bus_lines",
                        "business_secretarial_schools",
                        "buying_shopping_services",
                        "cable_satellite_and_other_pay_television_and_radio",
                        "camera_and_photographic_supply_stores",
                        "candy_nut_and_confectionery_stores",
                        "car_and_truck_dealers_new_used",
                        "car_and_truck_dealers_used_only",
                        "car_rental_agencies",
                        "car_washes",
                        "carpentry_services",
                        "carpet_upholstery_cleaning",
                        "caterers",
                        "charitable_and_social_service_organizations_fundraising",
                        "chemicals_and_allied_products",
                        "child_care_services",
                        "childrens_and_infants_wear_stores",
                        "chiropodists_podiatrists",
                        "chiropractors",
                        "cigar_stores_and_stands",
                        "civic_social_fraternal_associations",
                        "cleaning_and_maintenance",
                        "clothing_rental",
                        "colleges_universities",
                        "commercial_equipment",
                        "commercial_footwear",
                        "commercial_photography_art_and_graphics",
                        "commuter_transport_and_ferries",
                        "computer_network_services",
                        "computer_programming",
                        "computer_repair",
                        "computer_software_stores",
                        "computers_peripherals_and_software",
                        "concrete_work_services",
                        "construction_materials",
                        "consulting_public_relations",
                        "correspondence_schools",
                        "cosmetic_stores",
                        "counseling_services",
                        "country_clubs",
                        "courier_services",
                        "court_costs",
                        "credit_reporting_agencies",
                        "cruise_lines",
                        "dairy_products_stores",
                        "dance_hall_studios_schools",
                        "dating_escort_services",
                        "dentists_orthodontists",
                        "department_stores",
                        "detective_agencies",
                        "digital_goods_applications",
                        "digital_goods_games",
                        "digital_goods_large_volume",
                        "digital_goods_media",
                        "direct_marketing_catalog_merchant",
                        "direct_marketing_combination_catalog_and_retail_merchant",
                        "direct_marketing_inbound_telemarketing",
                        "direct_marketing_insurance_services",
                        "direct_marketing_other",
                        "direct_marketing_outbound_telemarketing",
                        "direct_marketing_subscription",
                        "direct_marketing_travel",
                        "discount_stores",
                        "doctors",
                        "door_to_door_sales",
                        "drapery_window_covering_and_upholstery_stores",
                        "drinking_places",
                        "drug_stores_and_pharmacies",
                        "drugs_drug_proprietaries_and_druggist_sundries",
                        "dry_cleaners",
                        "durable_goods",
                        "duty_free_stores",
                        "eating_places_restaurants",
                        "educational_services",
                        "electric_razor_stores",
                        "electric_vehicle_charging",
                        "electrical_parts_and_equipment",
                        "electrical_services",
                        "electronics_repair_shops",
                        "electronics_stores",
                        "elementary_secondary_schools",
                        "emergency_services_gcas_visa_use_only",
                        "employment_temp_agencies",
                        "equipment_rental",
                        "exterminating_services",
                        "family_clothing_stores",
                        "fast_food_restaurants",
                        "financial_institutions",
                        "fines_government_administrative_entities",
                        "fireplace_fireplace_screens_and_accessories_stores",
                        "floor_covering_stores",
                        "florists",
                        "florists_supplies_nursery_stock_and_flowers",
                        "freezer_and_locker_meat_provisioners",
                        "fuel_dealers_non_automotive",
                        "funeral_services_crematories",
                        "furniture_home_furnishings_and_equipment_stores_except_appliances",
                        "furniture_repair_refinishing",
                        "furriers_and_fur_shops",
                        "general_services",
                        "gift_card_novelty_and_souvenir_shops",
                        "glass_paint_and_wallpaper_stores",
                        "glassware_crystal_stores",
                        "golf_courses_public",
                        "government_licensed_horse_dog_racing_us_region_only",
                        "government_licensed_online_casions_online_gambling_us_region_only",
                        "government_owned_lotteries_non_us_region",
                        "government_owned_lotteries_us_region_only",
                        "government_services",
                        "grocery_stores_supermarkets",
                        "hardware_equipment_and_supplies",
                        "hardware_stores",
                        "health_and_beauty_spas",
                        "hearing_aids_sales_and_supplies",
                        "heating_plumbing_a_c",
                        "hobby_toy_and_game_shops",
                        "home_supply_warehouse_stores",
                        "hospitals",
                        "hotels_motels_and_resorts",
                        "household_appliance_stores",
                        "industrial_supplies",
                        "information_retrieval_services",
                        "insurance_default",
                        "insurance_underwriting_premiums",
                        "intra_company_purchases",
                        "jewelry_stores_watches_clocks_and_silverware_stores",
                        "landscaping_services",
                        "laundries",
                        "laundry_cleaning_services",
                        "legal_services_attorneys",
                        "luggage_and_leather_goods_stores",
                        "lumber_building_materials_stores",
                        "manual_cash_disburse",
                        "marinas_service_and_supplies",
                        "marketplaces",
                        "masonry_stonework_and_plaster",
                        "massage_parlors",
                        "medical_and_dental_labs",
                        "medical_dental_ophthalmic_and_hospital_equipment_and_supplies",
                        "medical_services",
                        "membership_organizations",
                        "mens_and_boys_clothing_and_accessories_stores",
                        "mens_womens_clothing_stores",
                        "metal_service_centers",
                        "miscellaneous",
                        "miscellaneous_apparel_and_accessory_shops",
                        "miscellaneous_auto_dealers",
                        "miscellaneous_business_services",
                        "miscellaneous_food_stores",
                        "miscellaneous_general_merchandise",
                        "miscellaneous_general_services",
                        "miscellaneous_home_furnishing_specialty_stores",
                        "miscellaneous_publishing_and_printing",
                        "miscellaneous_recreation_services",
                        "miscellaneous_repair_shops",
                        "miscellaneous_specialty_retail",
                        "mobile_home_dealers",
                        "motion_picture_theaters",
                        "motor_freight_carriers_and_trucking",
                        "motor_homes_dealers",
                        "motor_vehicle_supplies_and_new_parts",
                        "motorcycle_shops_and_dealers",
                        "motorcycle_shops_dealers",
                        "music_stores_musical_instruments_pianos_and_sheet_music",
                        "news_dealers_and_newsstands",
                        "non_fi_money_orders",
                        "non_fi_stored_value_card_purchase_load",
                        "nondurable_goods",
                        "nurseries_lawn_and_garden_supply_stores",
                        "nursing_personal_care",
                        "office_and_commercial_furniture",
                        "opticians_eyeglasses",
                        "optometrists_ophthalmologist",
                        "orthopedic_goods_prosthetic_devices",
                        "osteopaths",
                        "package_stores_beer_wine_and_liquor",
                        "paints_varnishes_and_supplies",
                        "parking_lots_garages",
                        "passenger_railways",
                        "pawn_shops",
                        "pet_shops_pet_food_and_supplies",
                        "petroleum_and_petroleum_products",
                        "photo_developing",
                        "photographic_photocopy_microfilm_equipment_and_supplies",
                        "photographic_studios",
                        "picture_video_production",
                        "piece_goods_notions_and_other_dry_goods",
                        "plumbing_heating_equipment_and_supplies",
                        "political_organizations",
                        "postal_services_government_only",
                        "precious_stones_and_metals_watches_and_jewelry",
                        "professional_services",
                        "public_warehousing_and_storage",
                        "quick_copy_repro_and_blueprint",
                        "railroads",
                        "real_estate_agents_and_managers_rentals",
                        "record_stores",
                        "recreational_vehicle_rentals",
                        "religious_goods_stores",
                        "religious_organizations",
                        "roofing_siding_sheet_metal",
                        "secretarial_support_services",
                        "security_brokers_dealers",
                        "service_stations",
                        "sewing_needlework_fabric_and_piece_goods_stores",
                        "shoe_repair_hat_cleaning",
                        "shoe_stores",
                        "small_appliance_repair",
                        "snowmobile_dealers",
                        "special_trade_services",
                        "specialty_cleaning",
                        "sporting_goods_stores",
                        "sporting_recreation_camps",
                        "sports_and_riding_apparel_stores",
                        "sports_clubs_fields",
                        "stamp_and_coin_stores",
                        "stationary_office_supplies_printing_and_writing_paper",
                        "stationery_stores_office_and_school_supply_stores",
                        "swimming_pools_sales",
                        "t_ui_travel_germany",
                        "tailors_alterations",
                        "tax_payments_government_agencies",
                        "tax_preparation_services",
                        "taxicabs_limousines",
                        "telecommunication_equipment_and_telephone_sales",
                        "telecommunication_services",
                        "telegraph_services",
                        "tent_and_awning_shops",
                        "testing_laboratories",
                        "theatrical_ticket_agencies",
                        "timeshares",
                        "tire_retreading_and_repair",
                        "tolls_bridge_fees",
                        "tourist_attractions_and_exhibits",
                        "towing_services",
                        "trailer_parks_campgrounds",
                        "transportation_services",
                        "travel_agencies_tour_operators",
                        "truck_stop_iteration",
                        "truck_utility_trailer_rentals",
                        "typesetting_plate_making_and_related_services",
                        "typewriter_stores",
                        "u_s_federal_government_agencies_or_departments",
                        "uniforms_commercial_clothing",
                        "used_merchandise_and_secondhand_stores",
                        "utilities",
                        "variety_stores",
                        "veterinary_services",
                        "video_amusement_game_supplies",
                        "video_game_arcades",
                        "video_tape_rental_stores",
                        "vocational_trade_schools",
                        "watch_jewelry_repair",
                        "welding_repair",
                        "wholesale_clubs",
                        "wig_and_toupee_stores",
                        "wires_money_orders",
                        "womens_accessory_and_specialty_shops",
                        "womens_ready_to_wear_stores",
                        "wrecking_and_salvage_yards",
                    ],
                    str,
                ]
            ]
        ]
        """
        Array of strings containing [categories](https://docs.stripe.com/api#issuing_authorization_object-merchant_data-category) of authorizations to allow. All other categories will be blocked. Cannot be set with `blocked_categories`.
        """
        allowed_merchant_countries: Optional[List[str]]
        """
        Array of strings containing representing countries from which authorizations will be allowed. Authorizations from merchants in all other countries will be declined. Country codes should be ISO 3166 alpha-2 country codes (e.g. `US`). Cannot be set with `blocked_merchant_countries`. Provide an empty value to unset this control.
        """
        blocked_card_presences: Optional[
            List[Union[Literal["not_present", "present"], str]]
        ]
        """
        Array of card presence statuses from which authorizations will be declined. Possible options are `present`, `not_present`. Cannot be set with `allowed_card_presences`. Provide an empty value to unset this control.
        """
        blocked_categories: Optional[
            List[
                Union[
                    Literal[
                        "ac_refrigeration_repair",
                        "accounting_bookkeeping_services",
                        "advertising_services",
                        "agricultural_cooperative",
                        "airlines_air_carriers",
                        "airports_flying_fields",
                        "ambulance_services",
                        "amusement_parks_carnivals",
                        "antique_reproductions",
                        "antique_shops",
                        "aquariums",
                        "architectural_surveying_services",
                        "art_dealers_and_galleries",
                        "artists_supply_and_craft_shops",
                        "auto_and_home_supply_stores",
                        "auto_body_repair_shops",
                        "auto_paint_shops",
                        "auto_service_shops",
                        "automated_cash_disburse",
                        "automated_fuel_dispensers",
                        "automobile_associations",
                        "automotive_parts_and_accessories_stores",
                        "automotive_tire_stores",
                        "bail_and_bond_payments",
                        "bakeries",
                        "bands_orchestras",
                        "barber_and_beauty_shops",
                        "betting_casino_gambling",
                        "bicycle_shops",
                        "billiard_pool_establishments",
                        "boat_dealers",
                        "boat_rentals_and_leases",
                        "book_stores",
                        "books_periodicals_and_newspapers",
                        "bowling_alleys",
                        "bus_lines",
                        "business_secretarial_schools",
                        "buying_shopping_services",
                        "cable_satellite_and_other_pay_television_and_radio",
                        "camera_and_photographic_supply_stores",
                        "candy_nut_and_confectionery_stores",
                        "car_and_truck_dealers_new_used",
                        "car_and_truck_dealers_used_only",
                        "car_rental_agencies",
                        "car_washes",
                        "carpentry_services",
                        "carpet_upholstery_cleaning",
                        "caterers",
                        "charitable_and_social_service_organizations_fundraising",
                        "chemicals_and_allied_products",
                        "child_care_services",
                        "childrens_and_infants_wear_stores",
                        "chiropodists_podiatrists",
                        "chiropractors",
                        "cigar_stores_and_stands",
                        "civic_social_fraternal_associations",
                        "cleaning_and_maintenance",
                        "clothing_rental",
                        "colleges_universities",
                        "commercial_equipment",
                        "commercial_footwear",
                        "commercial_photography_art_and_graphics",
                        "commuter_transport_and_ferries",
                        "computer_network_services",
                        "computer_programming",
                        "computer_repair",
                        "computer_software_stores",
                        "computers_peripherals_and_software",
                        "concrete_work_services",
                        "construction_materials",
                        "consulting_public_relations",
                        "correspondence_schools",
                        "cosmetic_stores",
                        "counseling_services",
                        "country_clubs",
                        "courier_services",
                        "court_costs",
                        "credit_reporting_agencies",
                        "cruise_lines",
                        "dairy_products_stores",
                        "dance_hall_studios_schools",
                        "dating_escort_services",
                        "dentists_orthodontists",
                        "department_stores",
                        "detective_agencies",
                        "digital_goods_applications",
                        "digital_goods_games",
                        "digital_goods_large_volume",
                        "digital_goods_media",
                        "direct_marketing_catalog_merchant",
                        "direct_marketing_combination_catalog_and_retail_merchant",
                        "direct_marketing_inbound_telemarketing",
                        "direct_marketing_insurance_services",
                        "direct_marketing_other",
                        "direct_marketing_outbound_telemarketing",
                        "direct_marketing_subscription",
                        "direct_marketing_travel",
                        "discount_stores",
                        "doctors",
                        "door_to_door_sales",
                        "drapery_window_covering_and_upholstery_stores",
                        "drinking_places",
                        "drug_stores_and_pharmacies",
                        "drugs_drug_proprietaries_and_druggist_sundries",
                        "dry_cleaners",
                        "durable_goods",
                        "duty_free_stores",
                        "eating_places_restaurants",
                        "educational_services",
                        "electric_razor_stores",
                        "electric_vehicle_charging",
                        "electrical_parts_and_equipment",
                        "electrical_services",
                        "electronics_repair_shops",
                        "electronics_stores",
                        "elementary_secondary_schools",
                        "emergency_services_gcas_visa_use_only",
                        "employment_temp_agencies",
                        "equipment_rental",
                        "exterminating_services",
                        "family_clothing_stores",
                        "fast_food_restaurants",
                        "financial_institutions",
                        "fines_government_administrative_entities",
                        "fireplace_fireplace_screens_and_accessories_stores",
                        "floor_covering_stores",
                        "florists",
                        "florists_supplies_nursery_stock_and_flowers",
                        "freezer_and_locker_meat_provisioners",
                        "fuel_dealers_non_automotive",
                        "funeral_services_crematories",
                        "furniture_home_furnishings_and_equipment_stores_except_appliances",
                        "furniture_repair_refinishing",
                        "furriers_and_fur_shops",
                        "general_services",
                        "gift_card_novelty_and_souvenir_shops",
                        "glass_paint_and_wallpaper_stores",
                        "glassware_crystal_stores",
                        "golf_courses_public",
                        "government_licensed_horse_dog_racing_us_region_only",
                        "government_licensed_online_casions_online_gambling_us_region_only",
                        "government_owned_lotteries_non_us_region",
                        "government_owned_lotteries_us_region_only",
                        "government_services",
                        "grocery_stores_supermarkets",
                        "hardware_equipment_and_supplies",
                        "hardware_stores",
                        "health_and_beauty_spas",
                        "hearing_aids_sales_and_supplies",
                        "heating_plumbing_a_c",
                        "hobby_toy_and_game_shops",
                        "home_supply_warehouse_stores",
                        "hospitals",
                        "hotels_motels_and_resorts",
                        "household_appliance_stores",
                        "industrial_supplies",
                        "information_retrieval_services",
                        "insurance_default",
                        "insurance_underwriting_premiums",
                        "intra_company_purchases",
                        "jewelry_stores_watches_clocks_and_silverware_stores",
                        "landscaping_services",
                        "laundries",
                        "laundry_cleaning_services",
                        "legal_services_attorneys",
                        "luggage_and_leather_goods_stores",
                        "lumber_building_materials_stores",
                        "manual_cash_disburse",
                        "marinas_service_and_supplies",
                        "marketplaces",
                        "masonry_stonework_and_plaster",
                        "massage_parlors",
                        "medical_and_dental_labs",
                        "medical_dental_ophthalmic_and_hospital_equipment_and_supplies",
                        "medical_services",
                        "membership_organizations",
                        "mens_and_boys_clothing_and_accessories_stores",
                        "mens_womens_clothing_stores",
                        "metal_service_centers",
                        "miscellaneous",
                        "miscellaneous_apparel_and_accessory_shops",
                        "miscellaneous_auto_dealers",
                        "miscellaneous_business_services",
                        "miscellaneous_food_stores",
                        "miscellaneous_general_merchandise",
                        "miscellaneous_general_services",
                        "miscellaneous_home_furnishing_specialty_stores",
                        "miscellaneous_publishing_and_printing",
                        "miscellaneous_recreation_services",
                        "miscellaneous_repair_shops",
                        "miscellaneous_specialty_retail",
                        "mobile_home_dealers",
                        "motion_picture_theaters",
                        "motor_freight_carriers_and_trucking",
                        "motor_homes_dealers",
                        "motor_vehicle_supplies_and_new_parts",
                        "motorcycle_shops_and_dealers",
                        "motorcycle_shops_dealers",
                        "music_stores_musical_instruments_pianos_and_sheet_music",
                        "news_dealers_and_newsstands",
                        "non_fi_money_orders",
                        "non_fi_stored_value_card_purchase_load",
                        "nondurable_goods",
                        "nurseries_lawn_and_garden_supply_stores",
                        "nursing_personal_care",
                        "office_and_commercial_furniture",
                        "opticians_eyeglasses",
                        "optometrists_ophthalmologist",
                        "orthopedic_goods_prosthetic_devices",
                        "osteopaths",
                        "package_stores_beer_wine_and_liquor",
                        "paints_varnishes_and_supplies",
                        "parking_lots_garages",
                        "passenger_railways",
                        "pawn_shops",
                        "pet_shops_pet_food_and_supplies",
                        "petroleum_and_petroleum_products",
                        "photo_developing",
                        "photographic_photocopy_microfilm_equipment_and_supplies",
                        "photographic_studios",
                        "picture_video_production",
                        "piece_goods_notions_and_other_dry_goods",
                        "plumbing_heating_equipment_and_supplies",
                        "political_organizations",
                        "postal_services_government_only",
                        "precious_stones_and_metals_watches_and_jewelry",
                        "professional_services",
                        "public_warehousing_and_storage",
                        "quick_copy_repro_and_blueprint",
                        "railroads",
                        "real_estate_agents_and_managers_rentals",
                        "record_stores",
                        "recreational_vehicle_rentals",
                        "religious_goods_stores",
                        "religious_organizations",
                        "roofing_siding_sheet_metal",
                        "secretarial_support_services",
                        "security_brokers_dealers",
                        "service_stations",
                        "sewing_needlework_fabric_and_piece_goods_stores",
                        "shoe_repair_hat_cleaning",
                        "shoe_stores",
                        "small_appliance_repair",
                        "snowmobile_dealers",
                        "special_trade_services",
                        "specialty_cleaning",
                        "sporting_goods_stores",
                        "sporting_recreation_camps",
                        "sports_and_riding_apparel_stores",
                        "sports_clubs_fields",
                        "stamp_and_coin_stores",
                        "stationary_office_supplies_printing_and_writing_paper",
                        "stationery_stores_office_and_school_supply_stores",
                        "swimming_pools_sales",
                        "t_ui_travel_germany",
                        "tailors_alterations",
                        "tax_payments_government_agencies",
                        "tax_preparation_services",
                        "taxicabs_limousines",
                        "telecommunication_equipment_and_telephone_sales",
                        "telecommunication_services",
                        "telegraph_services",
                        "tent_and_awning_shops",
                        "testing_laboratories",
                        "theatrical_ticket_agencies",
                        "timeshares",
                        "tire_retreading_and_repair",
                        "tolls_bridge_fees",
                        "tourist_attractions_and_exhibits",
                        "towing_services",
                        "trailer_parks_campgrounds",
                        "transportation_services",
                        "travel_agencies_tour_operators",
                        "truck_stop_iteration",
                        "truck_utility_trailer_rentals",
                        "typesetting_plate_making_and_related_services",
                        "typewriter_stores",
                        "u_s_federal_government_agencies_or_departments",
                        "uniforms_commercial_clothing",
                        "used_merchandise_and_secondhand_stores",
                        "utilities",
                        "variety_stores",
                        "veterinary_services",
                        "video_amusement_game_supplies",
                        "video_game_arcades",
                        "video_tape_rental_stores",
                        "vocational_trade_schools",
                        "watch_jewelry_repair",
                        "welding_repair",
                        "wholesale_clubs",
                        "wig_and_toupee_stores",
                        "wires_money_orders",
                        "womens_accessory_and_specialty_shops",
                        "womens_ready_to_wear_stores",
                        "wrecking_and_salvage_yards",
                    ],
                    str,
                ]
            ]
        ]
        """
        Array of strings containing [categories](https://docs.stripe.com/api#issuing_authorization_object-merchant_data-category) of authorizations to decline. All other categories will be allowed. Cannot be set with `allowed_categories`.
        """
        blocked_merchant_countries: Optional[List[str]]
        """
        Array of strings containing representing countries from which authorizations will be declined. Country codes should be ISO 3166 alpha-2 country codes (e.g. `US`). Cannot be set with `allowed_merchant_countries`. Provide an empty value to unset this control.
        """
        spending_limits: Optional[List[SpendingLimit]]
        """
        Limit spending with amount-based rules that apply across any cards this card replaced (i.e., its `replacement_for` card and _that_ card's `replacement_for` card, up the chain).
        """
        spending_limits_currency: Optional[str]
        """
        Currency of the amounts within `spending_limits`. Always the same as the currency of the card.
        """
        _inner_class_types = {"spending_limits": SpendingLimit}

    class Wallets(StripeObject):
        class ApplePay(StripeObject):
            eligible: bool
            """
            Apple Pay Eligibility
            """
            ineligible_reason: Optional[
                Union[
                    Literal[
                        "missing_agreement",
                        "missing_cardholder_contact",
                        "unsupported_region",
                    ],
                    str,
                ]
            ]
            """
            Reason the card is ineligible for Apple Pay
            """

        class GooglePay(StripeObject):
            eligible: bool
            """
            Google Pay Eligibility
            """
            ineligible_reason: Optional[
                Union[
                    Literal[
                        "missing_agreement",
                        "missing_cardholder_contact",
                        "unsupported_region",
                    ],
                    str,
                ]
            ]
            """
            Reason the card is ineligible for Google Pay
            """

        apple_pay: ApplePay
        google_pay: GooglePay
        primary_account_identifier: Optional[str]
        """
        Unique identifier for a card used with digital wallets
        """
        _inner_class_types = {"apple_pay": ApplePay, "google_pay": GooglePay}

    brand: str
    """
    The brand of the card.
    """
    cancellation_reason: Optional[
        Literal["design_rejected", "fulfillment_error", "lost", "stolen"]
    ]
    """
    The reason why the card was canceled.
    """
    cardholder: "Cardholder"
    """
    An Issuing `Cardholder` object represents an individual or business entity who is [issued](https://docs.stripe.com/issuing) cards.

    Related guide: [How to create a cardholder](https://docs.stripe.com/issuing/cards/virtual/issue-cards#create-cardholder)
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Supported currencies are `usd` in the US, `eur` in the EU, and `gbp` in the UK.
    """
    cvc: Optional[str]
    """
    The card's CVC. For security reasons, this is only available for virtual cards, and will be omitted unless you explicitly request it with [the `expand` parameter](https://docs.stripe.com/api/expanding_objects). Additionally, it's only available via the ["Retrieve a card" endpoint](https://docs.stripe.com/api/issuing/cards/retrieve), not via "List all cards" or any other endpoint.
    """
    exp_month: int
    """
    The expiration month of the card.
    """
    exp_year: int
    """
    The expiration year of the card.
    """
    financial_account: Optional[str]
    """
    The financial account this card is attached to.
    """
    id: str
    """
    Unique identifier for the object.
    """
    last4: str
    """
    The last 4 digits of the card number.
    """
    latest_fraud_warning: Optional[LatestFraudWarning]
    """
    Stripe's assessment of whether this card's details have been compromised. If this property isn't null, cancel and reissue the card to prevent fraudulent activity risk.
    """
    lifecycle_controls: Optional[LifecycleControls]
    """
    Rules that control the lifecycle of this card, such as automatic cancellation. Refer to our [documentation](https://docs.stripe.com/issuing/controls/lifecycle-controls) for more details.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: UntypedStripeObject[str]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    number: Optional[str]
    """
    The full unredacted card number. For security reasons, this is only available for virtual cards, and will be omitted unless you explicitly request it with [the `expand` parameter](https://docs.stripe.com/api/expanding_objects). Additionally, it's only available via the ["Retrieve a card" endpoint](https://docs.stripe.com/api/issuing/cards/retrieve), not via "List all cards" or any other endpoint.
    """
    object: Literal["issuing.card"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    personalization_design: Optional[ExpandableField["PersonalizationDesign"]]
    """
    The personalization design object belonging to this card.
    """
    replaced_by: Optional[ExpandableField["Card"]]
    """
    The latest card that replaces this card, if any.
    """
    replacement_for: Optional[ExpandableField["Card"]]
    """
    The card this card replaces, if any.
    """
    replacement_reason: Optional[
        Literal["damaged", "expired", "fulfillment_error", "lost", "stolen"]
    ]
    """
    The reason why the previous card needed to be replaced.
    """
    second_line: Optional[str]
    """
    Text separate from cardholder name, printed on the card.
    """
    shipping: Optional[Shipping]
    """
    Where and how the card will be shipped.
    """
    spending_controls: SpendingControls
    status: Union[Literal["active", "canceled", "inactive"], str]
    """
    Whether authorizations can be approved on this card. May be blocked from activating cards depending on past-due Cardholder requirements. Defaults to `inactive`.
    """
    type: Union[Literal["physical", "virtual"], str]
    """
    The type of the card.
    """
    wallets: Optional[Wallets]
    """
    Information relating to digital wallets (like Apple Pay and Google Pay).
    """

    @classmethod
    def create(cls, **params: Unpack["CardCreateParams"]) -> "Card":
        """
        Creates an Issuing Card object.
        """
        return cast(
            "Card",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["CardCreateParams"]
    ) -> "Card":
        """
        Creates an Issuing Card object.
        """
        return cast(
            "Card",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(cls, **params: Unpack["CardListParams"]) -> ListObject["Card"]:
        """
        Returns a list of Issuing Card objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["CardListParams"]
    ) -> ListObject["Card"]:
        """
        Returns a list of Issuing Card objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(cls, id: str, **params: Unpack["CardModifyParams"]) -> "Card":
        """
        Updates the specified Issuing Card object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Card",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["CardModifyParams"]
    ) -> "Card":
        """
        Updates the specified Issuing Card object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Card",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["CardRetrieveParams"]
    ) -> "Card":
        """
        Retrieves an Issuing Card object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["CardRetrieveParams"]
    ) -> "Card":
        """
        Retrieves an Issuing Card object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    class TestHelpers(APIResourceTestHelpers["Card"]):
        _resource_cls: Type["Card"]

        @classmethod
        def _cls_deliver_card(
            cls, card: str, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            return cast(
                "Card",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/deliver".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def deliver_card(
            card: str, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            ...

        @overload
        def deliver_card(
            self, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            ...

        @class_method_variant("_cls_deliver_card")
        def deliver_card(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            return cast(
                "Card",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/deliver".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_deliver_card_async(
            cls, card: str, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            return cast(
                "Card",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/deliver".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def deliver_card_async(
            card: str, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            ...

        @overload
        async def deliver_card_async(
            self, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            ...

        @class_method_variant("_cls_deliver_card_async")
        async def deliver_card_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardDeliverCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to delivered.
            """
            return cast(
                "Card",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/deliver".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_fail_card(
            cls, card: str, **params: Unpack["CardFailCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            return cast(
                "Card",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/fail".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def fail_card(
            card: str, **params: Unpack["CardFailCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            ...

        @overload
        def fail_card(self, **params: Unpack["CardFailCardParams"]) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            ...

        @class_method_variant("_cls_fail_card")
        def fail_card(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardFailCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            return cast(
                "Card",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/fail".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_fail_card_async(
            cls, card: str, **params: Unpack["CardFailCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            return cast(
                "Card",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/fail".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def fail_card_async(
            card: str, **params: Unpack["CardFailCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            ...

        @overload
        async def fail_card_async(
            self, **params: Unpack["CardFailCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            ...

        @class_method_variant("_cls_fail_card_async")
        async def fail_card_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardFailCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to failure.
            """
            return cast(
                "Card",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/fail".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_return_card(
            cls, card: str, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            return cast(
                "Card",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/return".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def return_card(
            card: str, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            ...

        @overload
        def return_card(
            self, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            ...

        @class_method_variant("_cls_return_card")
        def return_card(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            return cast(
                "Card",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/return".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_return_card_async(
            cls, card: str, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            return cast(
                "Card",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/return".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def return_card_async(
            card: str, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            ...

        @overload
        async def return_card_async(
            self, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            ...

        @class_method_variant("_cls_return_card_async")
        async def return_card_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardReturnCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to returned.
            """
            return cast(
                "Card",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/return".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_ship_card(
            cls, card: str, **params: Unpack["CardShipCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            return cast(
                "Card",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/ship".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def ship_card(
            card: str, **params: Unpack["CardShipCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            ...

        @overload
        def ship_card(self, **params: Unpack["CardShipCardParams"]) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            ...

        @class_method_variant("_cls_ship_card")
        def ship_card(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardShipCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            return cast(
                "Card",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/ship".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_ship_card_async(
            cls, card: str, **params: Unpack["CardShipCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            return cast(
                "Card",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/ship".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def ship_card_async(
            card: str, **params: Unpack["CardShipCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            ...

        @overload
        async def ship_card_async(
            self, **params: Unpack["CardShipCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            ...

        @class_method_variant("_cls_ship_card_async")
        async def ship_card_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardShipCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to shipped.
            """
            return cast(
                "Card",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/ship".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_submit_card(
            cls, card: str, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            return cast(
                "Card",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/submit".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def submit_card(
            card: str, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            ...

        @overload
        def submit_card(
            self, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            ...

        @class_method_variant("_cls_submit_card")
        def submit_card(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            return cast(
                "Card",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/submit".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_submit_card_async(
            cls, card: str, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            return cast(
                "Card",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/submit".format(
                        card=sanitize_id(card)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def submit_card_async(
            card: str, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            ...

        @overload
        async def submit_card_async(
            self, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            ...

        @class_method_variant("_cls_submit_card_async")
        async def submit_card_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["CardSubmitCardParams"]
        ) -> "Card":
            """
            Updates the shipping status of the specified Issuing Card object to submitted. This method requires Stripe Version ‘2024-09-30.acacia' or later.
            """
            return cast(
                "Card",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/cards/{card}/shipping/submit".format(
                        card=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

    @property
    def test_helpers(self):
        return self.TestHelpers(self)

    _inner_class_types = {
        "latest_fraud_warning": LatestFraudWarning,
        "lifecycle_controls": LifecycleControls,
        "shipping": Shipping,
        "spending_controls": SpendingControls,
        "wallets": Wallets,
    }


Card.TestHelpers._resource_cls = Card



from stripe._searchable_api_resource import SearchableAPIResource
from stripe._charge import Charge


class TestSearchableAPIResource(object):
    class MySearchable(SearchableAPIResource):
        OBJECT_NAME = "mysearchable"

        @classmethod
        def search(cls, *args, **kwargs):
            return cls._search(
                search_url="/v1/mysearchables/search", *args, **kwargs
            )

    def test_search(self, http_client_mock):
        path = "/v1/mysearchables/search"
        query_string = "query=currency%3A%22CAD%22"
        http_client_mock.stub_request(
            "get",
            path,
            query_string,
            '{"object": "search_result", "data": [{"object": "charge", "name": "jose"}, {"object": "charge", "name": "curly"}], "url": "/v1/charges", "has_more": false, "next_page": null}',
            rheaders={"request-id": "req_id"},
        )

        res = self.MySearchable.search(query='currency:"CAD"')
        http_client_mock.assert_requested(
            "get", path=path, query_string=query_string
        )
        assert len(res.data) == 2
        assert all(isinstance(obj, Charge) for obj in res.data)
        assert res.data[0].name == "jose"
        assert res.data[1].name == "curly"

        assert res.last_response is not None
        assert res.last_response.request_id == "req_id"

    def test_search_multiple_pages(self, http_client_mock):
        path = "/v1/mysearchables/search"
        query_string = 'query=currency:"CAD"'
        http_client_mock.stub_request(
            "get",
            path,
            query_string,
            '{"object": "search_result", "data": [{"object": "charge", "name": "jose"}, {"object": "charge", "name": "curly"}], "url": "/v1/charges", "has_more": true, "next_page": "next-page-token"}',
            rheaders={"request-id": "req_id"},
        )

        res = self.MySearchable.search(query='currency:"CAD"')
        http_client_mock.assert_requested(
            "get", path=path, query_string=query_string
        )

        assert res.next_page == "next-page-token"

        query_string = 'page=next-page-token&query=currency:"CAD"'
        http_client_mock.stub_request(
            "get",
            path,
            query_string,
            '{"object": "search_result", "data": [{"object": "charge", "name": "test"}], "url": "/v1/charges", "has_more": false, "next_page": null}',
            rheaders={"request-id": "req_id"},
        )
        res2 = self.MySearchable.search(
            query='currency:"CAD"', page=res.next_page
        )
        http_client_mock.assert_requested(
            "get",
            path=path,
            query_string=query_string,
        )

        assert len(res2.data) == 1
        assert all(isinstance(obj, Charge) for obj in res2.data)
        assert res2.data[0].name == "test"



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class CustomerUpdateParams(TypedDict):
    address: NotRequired["Literal['']|CustomerUpdateParamsAddress"]
    """
    The customer's address. Learn about [country-specific requirements for calculating tax](https://docs.stripe.com/invoicing/taxes?dashboard-or-api=dashboard#set-up-customer).
    """
    balance: NotRequired[int]
    """
    An integer amount in cents (or local equivalent) that represents the customer's current balance, which affect the customer's future invoices. A negative amount represents a credit that decreases the amount due on an invoice; a positive amount increases the amount due on an invoice.
    """
    business_name: NotRequired["Literal['']|str"]
    """
    The customer's business name. This may be up to *150 characters*.
    """
    cash_balance: NotRequired["CustomerUpdateParamsCashBalance"]
    """
    Balance information and default balance settings for this customer.
    """
    default_source: NotRequired[str]
    """
    If you are using payment methods created via the PaymentMethods API, see the [invoice_settings.default_payment_method](https://docs.stripe.com/api/customers/update#update_customer-invoice_settings-default_payment_method) parameter.

    Provide the ID of a payment source already attached to this customer to make it this customer's default payment source.

    If you want to add a new payment source and make it the default, see the [source](https://docs.stripe.com/api/customers/update#update_customer-source) property.
    """
    description: NotRequired[str]
    """
    An arbitrary string that you can attach to a customer object. It is displayed alongside the customer in the dashboard.
    """
    email: NotRequired[str]
    """
    Customer's email address. It's displayed alongside the customer in your dashboard and can be useful for searching and tracking. This may be up to *512 characters*.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    individual_name: NotRequired["Literal['']|str"]
    """
    The customer's full name. This may be up to *150 characters*.
    """
    invoice_prefix: NotRequired[str]
    """
    The prefix for the customer used to generate unique invoice numbers. Must be 3–12 uppercase letters or numbers.
    """
    invoice_settings: NotRequired["CustomerUpdateParamsInvoiceSettings"]
    """
    Default invoice settings for this customer.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: NotRequired[str]
    """
    The customer's full name or business name.
    """
    next_invoice_sequence: NotRequired[int]
    """
    The sequence to be used on the customer's next invoice. Defaults to 1.
    """
    phone: NotRequired[str]
    """
    The customer's phone number.
    """
    preferred_locales: NotRequired[List[str]]
    """
    Customer's preferred languages, ordered by preference.
    """
    shipping: NotRequired["Literal['']|CustomerUpdateParamsShipping"]
    """
    The customer's shipping information. Appears on invoices emailed to this customer.
    """
    source: NotRequired[str]
    tax: NotRequired["CustomerUpdateParamsTax"]
    """
    Tax details about the customer.
    """
    tax_exempt: NotRequired[
        "Literal['']|Literal['exempt', 'none', 'reverse']|str"
    ]
    """
    The customer's tax exemption. One of `none`, `exempt`, or `reverse`.
    """
    validate: NotRequired[bool]


class CustomerUpdateParamsAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    A freeform text field for the country. However, in order to activate some tax features, the format should be a two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """


class CustomerUpdateParamsCashBalance(TypedDict):
    settings: NotRequired["CustomerUpdateParamsCashBalanceSettings"]
    """
    Settings controlling the behavior of the customer's cash balance,
    such as reconciliation of funds received.
    """


class CustomerUpdateParamsCashBalanceSettings(TypedDict):
    reconciliation_mode: NotRequired[
        "Literal['automatic', 'manual', 'merchant_default']|str"
    ]
    """
    Controls how funds transferred by the customer are applied to payment intents and invoices. Valid options are `automatic`, `manual`, or `merchant_default`. For more information about these reconciliation modes, see [Reconciliation](https://docs.stripe.com/payments/customer-balance/reconciliation).
    """


class CustomerUpdateParamsInvoiceSettings(TypedDict):
    custom_fields: NotRequired[
        "Literal['']|List[CustomerUpdateParamsInvoiceSettingsCustomField]"
    ]
    """
    The list of up to 4 default custom fields to be displayed on invoices for this customer. When updating, pass an empty string to remove previously-defined fields.
    """
    default_payment_method: NotRequired[str]
    """
    ID of a payment method that's attached to the customer, to be used as the customer's default payment method for subscriptions and invoices.
    """
    footer: NotRequired[str]
    """
    Default footer to be displayed on invoices for this customer.
    """
    rendering_options: NotRequired[
        "Literal['']|CustomerUpdateParamsInvoiceSettingsRenderingOptions"
    ]
    """
    Default options for invoice PDF rendering for this customer.
    """


class CustomerUpdateParamsInvoiceSettingsCustomField(TypedDict):
    name: str
    """
    The name of the custom field. This may be up to 40 characters.
    """
    value: str
    """
    The value of the custom field. This may be up to 140 characters.
    """


class CustomerUpdateParamsInvoiceSettingsRenderingOptions(TypedDict):
    amount_tax_display: NotRequired[
        "Literal['']|Literal['exclude_tax', 'include_inclusive_tax']|str"
    ]
    """
    How line-item prices and amounts will be displayed with respect to tax on invoice PDFs. One of `exclude_tax` or `include_inclusive_tax`. `include_inclusive_tax` will include inclusive tax (and exclude exclusive tax) in invoice PDF amounts. `exclude_tax` will exclude all tax (inclusive and exclusive alike) from invoice PDF amounts.
    """
    template: NotRequired[str]
    """
    ID of the invoice rendering template to use for future invoices.
    """


class CustomerUpdateParamsShipping(TypedDict):
    address: "CustomerUpdateParamsShippingAddress"
    """
    Customer shipping address.
    """
    name: str
    """
    Customer name.
    """
    phone: NotRequired[str]
    """
    Customer phone (including extension).
    """


class CustomerUpdateParamsShippingAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    A freeform text field for the country. However, in order to activate some tax features, the format should be a two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """


class CustomerUpdateParamsTax(TypedDict):
    ip_address: NotRequired["Literal['']|str"]
    """
    A recent IP address of the customer used for tax reporting and tax location inference. Stripe recommends updating the IP address when a new PaymentMethod is attached or the address field on the customer is updated. We recommend against updating this field more frequently since it could result in unexpected tax location/reporting outcomes.
    """
    validate_location: NotRequired[
        "Literal['auto', 'deferred', 'immediately']|str"
    ]
    """
    A flag that indicates when Stripe should validate the customer tax location. Defaults to `auto`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from typing import ClassVar, Union
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.issuing._physical_bundle_list_params import (
        PhysicalBundleListParams,
    )
    from stripe.params.issuing._physical_bundle_retrieve_params import (
        PhysicalBundleRetrieveParams,
    )


class PhysicalBundle(ListableAPIResource["PhysicalBundle"]):
    """
    A Physical Bundle represents the bundle of physical items - card stock, carrier letter, and envelope - that is shipped to a cardholder when you create a physical card.
    """

    OBJECT_NAME: ClassVar[Literal["issuing.physical_bundle"]] = (
        "issuing.physical_bundle"
    )

    class Features(StripeObject):
        card_logo: Union[Literal["optional", "required", "unsupported"], str]
        """
        The policy for how to use card logo images in a card design with this physical bundle.
        """
        carrier_text: Union[
            Literal["optional", "required", "unsupported"], str
        ]
        """
        The policy for how to use carrier letter text in a card design with this physical bundle.
        """
        second_line: Union[Literal["optional", "required", "unsupported"], str]
        """
        The policy for how to use a second line on a card with this physical bundle.
        """

    features: Features
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    name: str
    """
    Friendly display name.
    """
    object: Literal["issuing.physical_bundle"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    status: Union[Literal["active", "inactive", "review"], str]
    """
    Whether this physical bundle can be used to create cards.
    """
    type: Union[Literal["custom", "standard"], str]
    """
    Whether this physical bundle is a standard Stripe offering or custom-made for you.
    """

    @classmethod
    def list(
        cls, **params: Unpack["PhysicalBundleListParams"]
    ) -> ListObject["PhysicalBundle"]:
        """
        Returns a list of physical bundle objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["PhysicalBundleListParams"]
    ) -> ListObject["PhysicalBundle"]:
        """
        Returns a list of physical bundle objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["PhysicalBundleRetrieveParams"]
    ) -> "PhysicalBundle":
        """
        Retrieves a physical bundle object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["PhysicalBundleRetrieveParams"]
    ) -> "PhysicalBundle":
        """
        Retrieves a physical bundle object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"features": Features}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._review import Review
    from stripe.params._review_approve_params import ReviewApproveParams
    from stripe.params._review_list_params import ReviewListParams
    from stripe.params._review_retrieve_params import ReviewRetrieveParams


class ReviewService(StripeService):
    def list(
        self,
        params: Optional["ReviewListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Review]":
        """
        Returns a list of Review objects that have open set to true. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            "ListObject[Review]",
            self._request(
                "get",
                "/v1/reviews",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ReviewListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Review]":
        """
        Returns a list of Review objects that have open set to true. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            "ListObject[Review]",
            await self._request_async(
                "get",
                "/v1/reviews",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        review: str,
        params: Optional["ReviewRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Review":
        """
        Retrieves a Review object.
        """
        return cast(
            "Review",
            self._request(
                "get",
                "/v1/reviews/{review}".format(review=sanitize_id(review)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        review: str,
        params: Optional["ReviewRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Review":
        """
        Retrieves a Review object.
        """
        return cast(
            "Review",
            await self._request_async(
                "get",
                "/v1/reviews/{review}".format(review=sanitize_id(review)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def approve(
        self,
        review: str,
        params: Optional["ReviewApproveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Review":
        """
        Approves a Review object, closing it and removing it from the list of reviews.
        """
        return cast(
            "Review",
            self._request(
                "post",
                "/v1/reviews/{review}/approve".format(
                    review=sanitize_id(review),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def approve_async(
        self,
        review: str,
        params: Optional["ReviewApproveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Review":
        """
        Approves a Review object, closing it and removing it from the list of reviews.
        """
        return cast(
            "Review",
            await self._request_async(
                "post",
                "/v1/reviews/{review}/approve".format(
                    review=sanitize_id(review),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.test_helpers.treasury._inbound_transfer_service import (
        InboundTransferService as InboundTransferService,
    )
    from stripe.test_helpers.treasury._outbound_payment_service import (
        OutboundPaymentService as OutboundPaymentService,
    )
    from stripe.test_helpers.treasury._outbound_transfer_service import (
        OutboundTransferService as OutboundTransferService,
    )
    from stripe.test_helpers.treasury._received_credit_service import (
        ReceivedCreditService as ReceivedCreditService,
    )
    from stripe.test_helpers.treasury._received_debit_service import (
        ReceivedDebitService as ReceivedDebitService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "InboundTransferService": (
        "stripe.test_helpers.treasury._inbound_transfer_service",
        False,
    ),
    "OutboundPaymentService": (
        "stripe.test_helpers.treasury._outbound_payment_service",
        False,
    ),
    "OutboundTransferService": (
        "stripe.test_helpers.treasury._outbound_transfer_service",
        False,
    ),
    "ReceivedCreditService": (
        "stripe.test_helpers.treasury._received_credit_service",
        False,
    ),
    "ReceivedDebitService": (
        "stripe.test_helpers.treasury._received_debit_service",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class VerificationSessionCreateParams(RequestOptions):
    client_reference_id: NotRequired[str]
    """
    A string to reference this user. This can be a customer ID, a session ID, or similar, and can be used to reconcile this verification with your internal systems.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    options: NotRequired["VerificationSessionCreateParamsOptions"]
    """
    A set of options for the session's verification checks.
    """
    provided_details: NotRequired[
        "VerificationSessionCreateParamsProvidedDetails"
    ]
    """
    Details provided about the user being verified. These details might be shown to the user.
    """
    related_customer: NotRequired[str]
    """
    Customer ID
    """
    related_customer_account: NotRequired[str]
    """
    The ID of the Account representing a customer.
    """
    related_person: NotRequired["VerificationSessionCreateParamsRelatedPerson"]
    """
    Tokens referencing a Person resource and its associated account.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon completing the verification flow.
    """
    type: NotRequired["Literal['document', 'id_number']|str"]
    """
    The type of [verification check](https://docs.stripe.com/identity/verification-checks) to be performed. You must provide a `type` if not passing `verification_flow`.
    """
    verification_flow: NotRequired[str]
    """
    The ID of a verification flow from the Dashboard. See https://docs.stripe.com/identity/verification-flows.
    """


class VerificationSessionCreateParamsOptions(TypedDict):
    document: NotRequired[
        "Literal['']|VerificationSessionCreateParamsOptionsDocument"
    ]
    """
    Options that apply to the [document check](https://docs.stripe.com/identity/verification-checks?type=document).
    """


class VerificationSessionCreateParamsOptionsDocument(TypedDict):
    allowed_types: NotRequired[
        List[Union[Literal["driving_license", "id_card", "passport"], str]]
    ]
    """
    Array of strings of allowed identity document types. If the provided identity document isn't one of the allowed types, the verification check will fail with a document_type_not_allowed error code.
    """
    require_id_number: NotRequired[bool]
    """
    Collect an ID number and perform an [ID number check](https://docs.stripe.com/identity/verification-checks?type=id-number) with the document's extracted name and date of birth.
    """
    require_live_capture: NotRequired[bool]
    """
    Disable image uploads, identity document images have to be captured using the device's camera.
    """
    require_matching_selfie: NotRequired[bool]
    """
    Capture a face image and perform a [selfie check](https://docs.stripe.com/identity/verification-checks?type=selfie) comparing a photo ID and a picture of your user's face. [Learn more](https://docs.stripe.com/identity/selfie).
    """


class VerificationSessionCreateParamsProvidedDetails(TypedDict):
    email: NotRequired[str]
    """
    Email of user being verified
    """
    phone: NotRequired[str]
    """
    Phone number of user being verified
    """


class VerificationSessionCreateParamsRelatedPerson(TypedDict):
    account: str
    """
    A token representing a connected account. If provided, the person parameter is also required and must be associated with the account.
    """
    person: str
    """
    A token referencing a Person resource that this verification is being used to verify.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class PaymentMethodDomainCreateParams(RequestOptions):
    domain_name: str
    """
    The domain name that this payment method domain object represents.
    """
    enabled: NotRequired[bool]
    """
    Whether this payment method domain is enabled. If the domain is not enabled, payment methods that require a payment method domain will not appear in Elements or Embedded Checkout.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """



from typing import TypeVar, Iterator, AsyncIterator

T = TypeVar("T")


class AnyIterator(Iterator[T], AsyncIterator[T]):
    """
    AnyIterator supports iteration through both `for ... in <AnyIterator>` and `async for ... in <AnyIterator> syntaxes.
    """

    def __init__(
        self, iterator: Iterator[T], async_iterator: AsyncIterator[T]
    ) -> None:
        self._iterator = iterator
        self._async_iterator = async_iterator

        self._sync_iterated = False
        self._async_iterated = False

    def __next__(self) -> T:
        if self._async_iterated:
            raise RuntimeError(
                "AnyIterator error: cannot mix sync and async iteration"
            )
        self._sync_iterated = True
        return self._iterator.__next__()

    async def __anext__(self) -> T:
        if self._sync_iterated:
            raise RuntimeError(
                "AnyIterator error: cannot mix sync and async iteration"
            )
        self._async_iterated = True
        return await self._async_iterator.__anext__()



"""
Tests for V2 decimal_string field encoding and decoding.

V2 API decimal fields are encoded as strings on the wire but exposed as
native Python Decimal values. These tests verify both directions:
- Request serialization: Decimal/int/float → JSON string
- Response hydration: JSON string → Decimal
"""

from decimal import Decimal

from stripe._encode import _coerce_decimal_string, _coerce_v2_params
from stripe._stripe_object import StripeObject


class TestCoerceDecimalString:
    """Tests for the shared bidirectional coercion helper."""

    def test_encode_decimal_to_str(self):
        assert _coerce_decimal_string(Decimal("1.5"), encode=True) == "1.5"

    def test_encode_int_to_str(self):
        assert _coerce_decimal_string(100, encode=True) == "100"

    def test_encode_float_to_str(self):
        assert _coerce_decimal_string(1.5, encode=True) == "1.5"

    def test_decode_str_to_decimal(self):
        result = _coerce_decimal_string("1.5", encode=False)
        assert result == Decimal("1.5")
        assert isinstance(result, Decimal)

    def test_encode_list(self):
        result = _coerce_decimal_string(
            [Decimal("1.1"), Decimal("2.2"), Decimal("3.3")], encode=True
        )
        assert result == ["1.1", "2.2", "3.3"]

    def test_decode_list(self):
        result = _coerce_decimal_string(["1.1", "2.2", "3.3"], encode=False)
        assert result == [Decimal("1.1"), Decimal("2.2"), Decimal("3.3")]
        assert all(isinstance(v, Decimal) for v in result)

    def test_none_passthrough(self):
        assert _coerce_decimal_string(None, encode=True) is None
        assert _coerce_decimal_string(None, encode=False) is None

    def test_bool_not_coerced_encode(self):
        """Booleans must not be coerced (bool is subclass of int)."""
        assert _coerce_decimal_string(True, encode=True) is True
        assert _coerce_decimal_string(False, encode=True) is False

    def test_already_str_passthrough_encode(self):
        """A string value passes through on encode (already serialized)."""
        assert _coerce_decimal_string("1.5", encode=True) == "1.5"

    def test_non_str_passthrough_decode(self):
        """Non-string values pass through on decode."""
        assert _coerce_decimal_string(42, encode=False) == 42


class TestDecimalStringPrecision:
    """Precision is preserved through the encode/decode round-trip."""

    def test_high_precision_encode(self):
        value = Decimal("1.23456789012345678901234567890")
        result = _coerce_decimal_string(value, encode=True)
        assert result == "1.23456789012345678901234567890"

    def test_high_precision_decode(self):
        result = _coerce_decimal_string(
            "1.23456789012345678901234567890", encode=False
        )
        assert result == Decimal("1.23456789012345678901234567890")

    def test_round_trip_preserves_precision(self):
        original = Decimal("9999999999999999.9999999999999999")
        encoded = _coerce_decimal_string(original, encode=True)
        decoded = _coerce_decimal_string(encoded, encode=False)
        assert decoded == original

    def test_zero(self):
        assert _coerce_decimal_string(Decimal("0"), encode=True) == "0"
        assert _coerce_decimal_string("0", encode=False) == Decimal("0")

    def test_negative(self):
        assert _coerce_decimal_string(Decimal("-1.5"), encode=True) == "-1.5"
        assert _coerce_decimal_string("-1.5", encode=False) == Decimal("-1.5")


class TestCoerceV2ParamsDecimalString:
    """Tests for outbound request coercion (Decimal → str)."""

    def test_top_level_decimal_string(self):
        params = {"price": Decimal("9.99")}
        schema = {"price": "decimal_string"}
        result = _coerce_v2_params(params, schema)
        assert result == {"price": "9.99"}

    def test_nested_decimal_string(self):
        params = {"item": {"price": Decimal("1.50")}}
        schema = {"item": {"price": "decimal_string"}}
        result = _coerce_v2_params(params, schema)
        assert result == {"item": {"price": "1.50"}}

    def test_array_of_decimal_string(self):
        params = {"prices": [Decimal("1.0"), Decimal("2.0"), Decimal("3.0")]}
        schema = {"prices": "decimal_string"}
        result = _coerce_v2_params(params, schema)
        assert result == {"prices": ["1.0", "2.0", "3.0"]}

    def test_array_of_objects_with_decimal_string(self):
        params = {
            "items": [{"price": Decimal("1.00")}, {"price": Decimal("2.00")}]
        }
        schema = {"items": {"price": "decimal_string"}}
        result = _coerce_v2_params(params, schema)
        assert result == {"items": [{"price": "1.00"}, {"price": "2.00"}]}

    def test_unrelated_fields_unchanged(self):
        params = {"name": "test", "count": 42, "price": Decimal("9.99")}
        schema = {"price": "decimal_string"}
        result = _coerce_v2_params(params, schema)
        assert result == {"name": "test", "count": 42, "price": "9.99"}

    def test_none_params_returns_none(self):
        schema = {"price": "decimal_string"}
        result = _coerce_v2_params(None, schema)
        assert result is None

    def test_none_value_preserved(self):
        params = {"price": None}
        schema = {"price": "decimal_string"}
        result = _coerce_v2_params(params, schema)
        assert result == {"price": None}

    def test_int_value_coerced(self):
        """int values are also accepted as decimal_string inputs."""
        params = {"price": 10}
        schema = {"price": "decimal_string"}
        result = _coerce_v2_params(params, schema)
        assert result == {"price": "10"}

    def test_mixed_decimal_and_int64(self):
        params = {"price": Decimal("9.99"), "quantity": 5}
        schema = {"price": "decimal_string", "quantity": "int64_string"}
        result = _coerce_v2_params(params, schema)
        assert result == {"price": "9.99", "quantity": "5"}


class TestResponseFieldCoercion:
    """Tests for inbound response coercion (str → Decimal) via _field_encodings."""

    def _make_v2_class(self, field_encodings):
        """Create a StripeObject subclass with given field encodings."""

        class V2Resource(StripeObject):
            _field_encodings = field_encodings

        return V2Resource

    def test_top_level_decimal_string_response(self):
        cls = self._make_v2_class({"price": "decimal_string"})
        obj = cls.construct_from(
            {"id": "test", "price": "9.99"},
            key="sk_test",
            api_mode="V2",
        )
        assert obj["price"] == Decimal("9.99")
        assert isinstance(obj["price"], Decimal)

    def test_array_of_decimal_string_response(self):
        cls = self._make_v2_class({"prices": "decimal_string"})
        obj = cls.construct_from(
            {"id": "test", "prices": ["1.0", "2.0", "3.0"]},
            key="sk_test",
            api_mode="V2",
        )
        assert obj["prices"] == [
            Decimal("1.0"),
            Decimal("2.0"),
            Decimal("3.0"),
        ]
        assert all(isinstance(v, Decimal) for v in obj["prices"])

    def test_unrelated_fields_unchanged_response(self):
        cls = self._make_v2_class({"price": "decimal_string"})
        obj = cls.construct_from(
            {"id": "test", "name": "hello", "price": "9.99"},
            key="sk_test",
            api_mode="V2",
        )
        assert obj["name"] == "hello"
        assert isinstance(obj["name"], str)
        assert obj["price"] == Decimal("9.99")

    def test_none_value_preserved_response(self):
        cls = self._make_v2_class({"price": "decimal_string"})
        obj = cls.construct_from(
            {"id": "test", "price": None},
            key="sk_test",
            api_mode="V2",
        )
        assert obj["price"] is None

    def test_no_field_encodings_unchanged(self):
        """StripeObject without _field_encodings leaves strings as strings."""
        obj = StripeObject.construct_from(
            {"id": "test", "price": "9.99"},
            key="sk_test",
            api_mode="V2",
        )
        assert obj["price"] == "9.99"
        assert isinstance(obj["price"], str)

    def test_high_precision_response(self):
        cls = self._make_v2_class({"rate": "decimal_string"})
        obj = cls.construct_from(
            {"id": "test", "rate": "0.00123456789012345678"},
            key="sk_test",
            api_mode="V2",
        )
        assert obj["rate"] == Decimal("0.00123456789012345678")

    def test_negative_response(self):
        cls = self._make_v2_class({"balance": "decimal_string"})
        obj = cls.construct_from(
            {"id": "test", "balance": "-100.50"},
            key="sk_test",
            api_mode="V2",
        )
        assert obj["balance"] == Decimal("-100.50")


class TestV1Unchanged:
    """Regression tests: V1 behavior should not be affected."""

    def test_v1_integer_fields_stay_integers(self):
        """V1 objects don't have decimal _field_encodings, so ints stay ints."""
        obj = StripeObject.construct_from(
            {"id": "test", "amount": 100},
            key="sk_test",
            api_mode="V1",
        )
        assert obj["amount"] == 100
        assert isinstance(obj["amount"], int)

    def test_v1_string_fields_stay_strings(self):
        obj = StripeObject.construct_from(
            {"id": "test", "name": "hello"},
            key="sk_test",
            api_mode="V1",
        )
        assert obj["name"] == "hello"
        assert isinstance(obj["name"], str)

    def test_object_without_field_encodings_unchanged(self):
        """Objects without decimal _field_encodings are unaffected."""
        obj = StripeObject.construct_from(
            {"id": "test", "price": "9.99", "count": 42},
            key="sk_test",
        )
        assert obj["price"] == "9.99"
        assert obj["count"] == 42



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._deletable_api_resource import DeletableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, Union, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._account import Account
    from stripe._application import Application
    from stripe._customer import Customer
    from stripe.params._tax_id_create_params import TaxIdCreateParams
    from stripe.params._tax_id_delete_params import TaxIdDeleteParams
    from stripe.params._tax_id_list_params import TaxIdListParams
    from stripe.params._tax_id_retrieve_params import TaxIdRetrieveParams


class TaxId(
    CreateableAPIResource["TaxId"],
    DeletableAPIResource["TaxId"],
    ListableAPIResource["TaxId"],
):
    """
    You can add one or multiple tax IDs to a [customer](https://docs.stripe.com/api/customers) or account.
    Customer and account tax IDs get displayed on related invoices and credit notes.

    Related guides: [Customer tax identification numbers](https://docs.stripe.com/billing/taxes/tax-ids), [Account tax IDs](https://docs.stripe.com/invoicing/connect#account-tax-ids)
    """

    OBJECT_NAME: ClassVar[Literal["tax_id"]] = "tax_id"

    class Owner(StripeObject):
        account: Optional[ExpandableField["Account"]]
        """
        The account being referenced when `type` is `account`.
        """
        application: Optional[ExpandableField["Application"]]
        """
        The Connect Application being referenced when `type` is `application`.
        """
        customer: Optional[ExpandableField["Customer"]]
        """
        The customer being referenced when `type` is `customer`.
        """
        customer_account: Optional[str]
        """
        The Account representing the customer being referenced when `type` is `customer`.
        """
        type: Union[Literal["account", "application", "customer", "self"], str]
        """
        Type of owner referenced.
        """

    class Verification(StripeObject):
        status: Union[
            Literal["pending", "unavailable", "unverified", "verified"], str
        ]
        """
        Verification status, one of `pending`, `verified`, `unverified`, or `unavailable`.
        """
        verified_address: Optional[str]
        """
        Verified address.
        """
        verified_name: Optional[str]
        """
        Verified name.
        """

    country: Optional[str]
    """
    Two-letter ISO code representing the country of the tax ID.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    customer: Optional[ExpandableField["Customer"]]
    """
    ID of the customer.
    """
    customer_account: Optional[str]
    """
    ID of the Account representing the customer.
    """
    deleted: Optional[Literal[True]]
    """
    Always true for a deleted object
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["tax_id"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    owner: Optional[Owner]
    """
    The account or customer the tax ID belongs to.
    """
    type: Union[
        Literal[
            "ad_nrt",
            "ae_trn",
            "al_tin",
            "am_tin",
            "ao_tin",
            "ar_cuit",
            "au_abn",
            "au_arn",
            "aw_tin",
            "az_tin",
            "ba_tin",
            "bb_tin",
            "bd_bin",
            "bf_ifu",
            "bg_uic",
            "bh_vat",
            "bj_ifu",
            "bo_tin",
            "br_cnpj",
            "br_cpf",
            "bs_tin",
            "by_tin",
            "ca_bn",
            "ca_gst_hst",
            "ca_pst_bc",
            "ca_pst_mb",
            "ca_pst_sk",
            "ca_qst",
            "cd_nif",
            "ch_uid",
            "ch_vat",
            "cl_tin",
            "cm_niu",
            "cn_tin",
            "co_nit",
            "cr_tin",
            "cv_nif",
            "de_stn",
            "do_rcn",
            "ec_ruc",
            "eg_tin",
            "es_cif",
            "et_tin",
            "eu_oss_vat",
            "eu_vat",
            "fo_vat",
            "gb_vat",
            "ge_vat",
            "gi_tin",
            "gn_nif",
            "hk_br",
            "hr_oib",
            "hu_tin",
            "ic_nif",
            "id_npwp",
            "il_vat",
            "in_gst",
            "is_vat",
            "it_cf",
            "jp_cn",
            "jp_rn",
            "jp_trn",
            "ke_pin",
            "kg_tin",
            "kh_tin",
            "kr_brn",
            "kz_bin",
            "la_tin",
            "li_uid",
            "li_vat",
            "lk_vat",
            "ma_vat",
            "md_vat",
            "me_pib",
            "mk_vat",
            "mr_nif",
            "mx_rfc",
            "my_frp",
            "my_itn",
            "my_sst",
            "ng_tin",
            "no_vat",
            "no_voec",
            "np_pan",
            "nz_gst",
            "om_vat",
            "pe_ruc",
            "ph_tin",
            "pl_nip",
            "py_ruc",
            "ro_tin",
            "rs_pib",
            "ru_inn",
            "ru_kpp",
            "sa_vat",
            "sg_gst",
            "sg_uen",
            "si_tin",
            "sn_ninea",
            "sr_fin",
            "sv_nit",
            "th_vat",
            "tj_tin",
            "tr_tin",
            "tw_vat",
            "tz_vat",
            "ua_vat",
            "ug_tin",
            "unknown",
            "us_ein",
            "uy_ruc",
            "uz_tin",
            "uz_vat",
            "ve_rif",
            "vn_tin",
            "za_vat",
            "zm_tin",
            "zw_tin",
        ],
        str,
    ]
    """
    Type of the tax ID, one of `ad_nrt`, `ae_trn`, `al_tin`, `am_tin`, `ao_tin`, `ar_cuit`, `au_abn`, `au_arn`, `aw_tin`, `az_tin`, `ba_tin`, `bb_tin`, `bd_bin`, `bf_ifu`, `bg_uic`, `bh_vat`, `bj_ifu`, `bo_tin`, `br_cnpj`, `br_cpf`, `bs_tin`, `by_tin`, `ca_bn`, `ca_gst_hst`, `ca_pst_bc`, `ca_pst_mb`, `ca_pst_sk`, `ca_qst`, `cd_nif`, `ch_uid`, `ch_vat`, `cl_tin`, `cm_niu`, `cn_tin`, `co_nit`, `cr_tin`, `cv_nif`, `de_stn`, `do_rcn`, `ec_ruc`, `eg_tin`, `es_cif`, `et_tin`, `eu_oss_vat`, `eu_vat`, `fo_vat`, `gb_vat`, `ge_vat`, `gi_tin`, `gn_nif`, `hk_br`, `hr_oib`, `hu_tin`, `ic_nif`, `id_npwp`, `il_vat`, `in_gst`, `is_vat`, `it_cf`, `jp_cn`, `jp_rn`, `jp_trn`, `ke_pin`, `kg_tin`, `kh_tin`, `kr_brn`, `kz_bin`, `la_tin`, `li_uid`, `li_vat`, `lk_vat`, `ma_vat`, `md_vat`, `me_pib`, `mk_vat`, `mr_nif`, `mx_rfc`, `my_frp`, `my_itn`, `my_sst`, `ng_tin`, `no_vat`, `no_voec`, `np_pan`, `nz_gst`, `om_vat`, `pe_ruc`, `ph_tin`, `pl_nip`, `py_ruc`, `ro_tin`, `rs_pib`, `ru_inn`, `ru_kpp`, `sa_vat`, `sg_gst`, `sg_uen`, `si_tin`, `sn_ninea`, `sr_fin`, `sv_nit`, `th_vat`, `tj_tin`, `tr_tin`, `tw_vat`, `tz_vat`, `ua_vat`, `ug_tin`, `us_ein`, `uy_ruc`, `uz_tin`, `uz_vat`, `ve_rif`, `vn_tin`, `za_vat`, `zm_tin`, or `zw_tin`. Note that some legacy tax IDs have type `unknown`
    """
    value: str
    """
    Value of the tax ID.
    """
    verification: Optional[Verification]
    """
    Tax ID verification information.
    """

    @classmethod
    def create(cls, **params: Unpack["TaxIdCreateParams"]) -> "TaxId":
        """
        Creates a new account or customer tax_id object.
        """
        return cast(
            "TaxId",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["TaxIdCreateParams"]
    ) -> "TaxId":
        """
        Creates a new account or customer tax_id object.
        """
        return cast(
            "TaxId",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_delete(
        cls, sid: str, **params: Unpack["TaxIdDeleteParams"]
    ) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "TaxId",
            cls._static_request(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    def delete(sid: str, **params: Unpack["TaxIdDeleteParams"]) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        ...

    @overload
    def delete(self, **params: Unpack["TaxIdDeleteParams"]) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        ...

    @class_method_variant("_cls_delete")
    def delete(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["TaxIdDeleteParams"]
    ) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        return self._request_and_refresh(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    async def _cls_delete_async(
        cls, sid: str, **params: Unpack["TaxIdDeleteParams"]
    ) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "TaxId",
            await cls._static_request_async(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def delete_async(
        sid: str, **params: Unpack["TaxIdDeleteParams"]
    ) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        ...

    @overload
    async def delete_async(
        self, **params: Unpack["TaxIdDeleteParams"]
    ) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        ...

    @class_method_variant("_cls_delete_async")
    async def delete_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["TaxIdDeleteParams"]
    ) -> "TaxId":
        """
        Deletes an existing account or customer tax_id object.
        """
        return await self._request_and_refresh_async(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    def list(cls, **params: Unpack["TaxIdListParams"]) -> ListObject["TaxId"]:
        """
        Returns a list of tax IDs.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["TaxIdListParams"]
    ) -> ListObject["TaxId"]:
        """
        Returns a list of tax IDs.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["TaxIdRetrieveParams"]
    ) -> "TaxId":
        """
        Retrieves an account or customer tax_id object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["TaxIdRetrieveParams"]
    ) -> "TaxId":
        """
        Retrieves an account or customer tax_id object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"owner": Owner, "verification": Verification}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.billing._alert_service import AlertService
    from stripe.billing._credit_balance_summary_service import (
        CreditBalanceSummaryService,
    )
    from stripe.billing._credit_balance_transaction_service import (
        CreditBalanceTransactionService,
    )
    from stripe.billing._credit_grant_service import CreditGrantService
    from stripe.billing._feedback_option_service import FeedbackOptionService
    from stripe.billing._meter_event_adjustment_service import (
        MeterEventAdjustmentService,
    )
    from stripe.billing._meter_event_service import MeterEventService
    from stripe.billing._meter_service import MeterService

_subservices = {
    "alerts": ["stripe.billing._alert_service", "AlertService"],
    "credit_balance_summary": [
        "stripe.billing._credit_balance_summary_service",
        "CreditBalanceSummaryService",
    ],
    "credit_balance_transactions": [
        "stripe.billing._credit_balance_transaction_service",
        "CreditBalanceTransactionService",
    ],
    "credit_grants": [
        "stripe.billing._credit_grant_service",
        "CreditGrantService",
    ],
    "feedback_options": [
        "stripe.billing._feedback_option_service",
        "FeedbackOptionService",
    ],
    "meters": ["stripe.billing._meter_service", "MeterService"],
    "meter_events": [
        "stripe.billing._meter_event_service",
        "MeterEventService",
    ],
    "meter_event_adjustments": [
        "stripe.billing._meter_event_adjustment_service",
        "MeterEventAdjustmentService",
    ],
}


class BillingService(StripeService):
    alerts: "AlertService"
    credit_balance_summary: "CreditBalanceSummaryService"
    credit_balance_transactions: "CreditBalanceTransactionService"
    credit_grants: "CreditGrantService"
    feedback_options: "FeedbackOptionService"
    meters: "MeterService"
    meter_events: "MeterEventService"
    meter_event_adjustments: "MeterEventAdjustmentService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()
